﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Xml;
using System.IO;

namespace Chie.NAntExecuter
{
  /// <summary>アプリケーションメインフォーム</summary>
  public partial class MainForm : Form
  {
    OutputWindow window = null;

    /// <summary>コンストラクタにまでコメントを要求しないでください</summary>
    /// <param name="args">コマンドライン。ここではだいいち引数のみを見る。それ以外の環境では素直にNAntを直接たたいて</param>
    public MainForm(string[] args)
    {
      InitializeComponent();
      if(args.Length > 0)
      {
        if(File.Exists(args[0]))
          fileName.FileName = args[0];
        else if(Directory.Exists(args[0]))
        {
          string path = "";
          if(Chie.Dialog.PromptForFileName(ref path, "", fileName.Filter, fileName.DefaultExt, args[0]))
            fileName.FileName = path;
        }
      }
      Application.Idle += new EventHandler(Application_Idle);
    }

    #region フォームイベントハンドラ

    /// <summary>アプリケーションアイドルイベント。ボタンのイネーブル変更</summary>
    void Application_Idle(object sender, EventArgs e)
    {
      openDirectory.Enabled = File.Exists(fileName.FileName);
      editButton.Enabled = openDirectory.Enabled;
      executeButton.Enabled = targets.SelectedItems.Count > 0;
    }

    /// <summary>フォームのアクティブイベント。ファイルを読み直す</summary>
    private void mainForm_Activated(object sender, EventArgs e)
    {
      fileName_TextChanged(null, null);
    }

    #endregion

    /// <summary>ファイル名変更。存在するファイルならそれを読み込みターゲットリスト表示。なければターゲットリストを消去して実行不可に</summary>
    private void fileName_TextChanged(object sender, EventArgs e)
    {
      targets.Items.Clear();
      if(File.Exists(fileName.FileName))
      {
        this.Refresh();
        this.SuspendLayout();
        try
        {
          // 読み込み
          XmlTextReader xml = new XmlTextReader(fileName.FileName);
          try
          {
            string defaultTarget = "";
            while(xml.Read())
            {
              if(xml.NodeType == XmlNodeType.Element)
              {
                if(xml.Name == "project" && xml.GetAttribute("default") != null){
                  defaultTarget = xml.GetAttribute("default");
                }else if(xml.Name == "target" && xml.GetAttribute("name") != null)
                {
                  // ターゲット追加
                  ListViewItem item = new ListViewItem();
                  item.Text = xml.GetAttribute("name");
                  if(xml.GetAttribute("description") != null)
                    item.SubItems.Add(xml.GetAttribute("description"));
                  if(defaultTarget == item.Text)
                    item.Selected = true;
                  targets.Items.Add(item);
                }
              }
            }
          }
          finally { xml.Close(); }
        }
        finally { this.ResumeLayout(true); }
      }
    }

    #region 操作ボタン

    /// <summary>開くボタンイベントハンドラ。Buildファイルのあるディレクトリをオープン</summary>
    private void openDirectory_Click(object sender, EventArgs e)
    {
      System.Diagnostics.Process.Start(Path.GetDirectoryName(fileName.FileName));
    }

    /// <summary>編集ボタンイベントハンドラ。Buildファイルをエディタで開く</summary>
    private void editButton_Click(object sender, EventArgs e)
    {
      string editorPath = Path.Combine(Path.GetDirectoryName(Application.ExecutablePath), "editor.lnk");
      if(File.Exists(editorPath))
        System.Diagnostics.Process.Start(editorPath, fileName.FileName);
      else
        System.Diagnostics.Process.Start("notepad", fileName.FileName);
    }

    /// <summary>実行ボタンイベントハンドラ。コマンドラインを生成。残りはnAntExecメソッドに任せる</summary>
    private void executeButton_Click(object sender, EventArgs e)
    {
      string cmd = string.Format("-nologo -buildfile:\"{0}\"", fileName.FileName);
      foreach(ListViewItem item in targets.SelectedItems)
      {
        cmd += string.Format(" \"{0}\"", item.Text);
      }

      nAntExec(cmd);      
    }

    /// <summary>終了ボタンイベントハンドラ。終了</summary>
    private void exitButton_Click(object sender, EventArgs e)
    {
      this.Close();
    }

    #endregion

    /// <summary>コマンドラインをNAntで実行する</summary>
    /// <param name="cmd">コマンドライン</param>
    private void nAntExec(string cmd)
    {
      // 起動処理
      System.Diagnostics.Process nant = new System.Diagnostics.Process();
      nant.StartInfo.FileName = "nant";
      nant.StartInfo.Arguments = cmd;
      nant.StartInfo.RedirectStandardOutput = true;
      nant.StartInfo.UseShellExecute = false;
      nant.StartInfo.WindowStyle = System.Diagnostics.ProcessWindowStyle.Hidden;
      nant.StartInfo.CreateNoWindow = true;
      // ログウィンドウ表示(もしあれば再利用する)
      if(window == null || !window.Visible) window = new OutputWindow();
      window.Show();
      window.BringToFront();
      window.ClearText();
      // START!!
      window.AppendText("-------- NAntを実行します --------" + Environment.NewLine);
      nant.Start();
      while(!nant.HasExited)
      {
        window.AppendText(nant.StandardOutput.ReadLine() + Environment.NewLine);
      }
      window.AppendText(nant.StandardOutput.ReadToEnd());
      window.AppendText("-------- 終了しました --------" + Environment.NewLine);
    }

  }
}